/*->c.Drawlevel1 */


#include "stdafx.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "os.h"
#include "bbc.h"
#include "sprite.h"
#include "wimp.h"
#include "font.h"

#include "transform.h"
#include "xprocess.h"
#include "xmath.h"
#include "colour.h"


#include "wos.h"


#define DRAWFILE_INTERNAL
#include "DrawFile1.h"
#include "DrawLevel0.h"
#include "DrawLevel1.h"
#include "DrawFile2.h"
#include "DrawErrors.h"




/*----------------------------- Global data ---------------------------------*/

/* Current mode state */
currentmodestr currentmode;

/* Flag needed for text areas */
BOOL Draw_memoryError;

/* Font reference translation table */
int Draw_transTable[256];
static BOOL fontTableChanged = FALSE;

/*------------------------------ Internal functions ----------------------------*/
/* These functions are local to the module */

/*
 Function    : dr_setStart
 Purpose     : set actual offset for start object
 Parameters  : start object handle
 Returns     : actual start offset
 Description : translates the special 'first object' code.
*/

static int dr_setStart(Draw_object start)
{
  return ((start == Draw_FirstObject) ? sizeof(draw_fileheader) : start);
}

/*
 Function    : dr_setEnd
 Purpose     : set actual end object
 Parameters  : diagram structure
               end object handle
               OUT: object header block
 Returns     : void
 Description : translates the 'end object' value. The offset returned is the
               offset in the diagram AFTER the last object. If the end object
               is not the special code, it is changed into an offset to the
               object following.
*/

static void dr_setEnd(Draw_diag * diag, Draw_object end, draw_objptr *hdrptr)
{
  if (end == Draw_LastObject)
    hdrptr->bytep = diag->data + diag->length;
  else
  {
    hdrptr->bytep = diag->data+end;
    hdrptr->bytep += hdrptr->objhdrp->size;
  }
}

/*
 Function    : dr_setBox
 Purpose     : set clipping box and origin (in draw units
 Parameters  : pointer to redraw structure
               scalefactor
               OUT: x, y origin
               OUT: clipping box
 Returns     : void
*/


/*
static void dr_setBox(Draw_redrawstr *r, double xscalefactor,
    double yscalefactor, int *orgx, int *orgy,double angle, Draw_box *clipBox)
{
 int x = r->box.x0 - r->scx;
 int y = r->box.y1 - r->scy;
 int     x0;
 int     x1;
 int     y0;
 int     y1;
 int     temp;
 double  ca;
 double  sa;

 *orgx = Draw_screenToDraw(x);
 *orgy = Draw_screenToDraw(y);

 clipBox->x0 = xscaledown(r->g.x0 - x);
 clipBox->y0 = yscaledown(r->g.y0 - y);
 clipBox->x1 = xscaledown(r->g.x1 - x);
 clipBox->y1 = yscaledown(r->g.y1 - y);

 if(angle)
 {
  ca=cos(-angle);
  sa=sin(-angle);

  x0=x1=(int)((double)clipBox->x0*ca-(double)clipBox->y0*sa);
  temp=(int)((double)clipBox->x1*ca-(double)clipBox->y1*sa);
  if(temp<x0) x0=temp;
  if(temp>x1) x1=temp;
  temp=(int)((double)clipBox->x0*ca-(double)clipBox->y1*sa);
  if(temp<x0) x0=temp; 
  if(temp>x1) x1=temp;
  temp=(int)((double)clipBox->x1*ca-(double)clipBox->y0*sa);
  if(temp<x0) x0=temp;
  if(temp>x1) x1=temp;

  y0=y1=(int)((double)clipBox->x0*sa+(double)clipBox->y0*ca);
  temp=(int)((double)clipBox->x1*sa+(double)clipBox->y1*ca);
  if(temp<y0) y0=temp;
  if(temp>y1) y1=temp;
  temp=(int)((double)clipBox->x1*sa+(double)clipBox->y0*ca);
  if(temp<y0) y0=temp;
  if(temp>y1) y1=temp;
  temp=(int)((double)clipBox->x0*sa+(double)clipBox->y1*ca);
  if(temp<y0) y0=temp;
  if(temp>y1) y1=temp;

  clipBox->x0=x0;
  clipBox->x1=x1;
  clipBox->y0=y0;
  clipBox->y1=y1;
 }
}
*/

/*
 Function    : dr_setScreenState
 Purpose     : cache mode variables and set up for output
 Parameters  : void
 Returns     : VDU status byte (as an int)
 Description : records the mode variables needed for rendering; records the VDU
               state and sets to VDU5 style output
*/

#ifdef NEVER

static int dr_setScreenState(void)
{
  int VDUstatus=0;           /* VDU status: only the VDU5 setting matters */

  currentmode.gcharaltered = 0;
  currentmode.pixsizex = cvdu.deltax << 8;
  currentmode.pixsizey = cvdu.deltay << 8;

  currentmode.gcharsizex=cvdu.gcharsizex;
  currentmode.gcharsizey=cvdu.gcharsizey;
  currentmode.gcharspacex=cvdu.gcharspacex;
  currentmode.gcharspacey=cvdu.gcharspacey;
 
  currentmode.ncolour=cvdu.ncolours;
  currentmode.xeigfactor=cvdu.lndeltax;
  currentmode.yeigfactor=cvdu.lndeltay;   


  /* Get VDU status */
/*  os_byte(117, &VDUstatus, 0); */

  /* Set to VDU5 mode */
/*  bbc_vdu(5); */

  return (VDUstatus);
}

/*
 Function    : dr_recoverScreenState
 Purpose     : restore (some) changed mode variables and reset the VDU5 state
 Parameters  : old vdu status
 Returns     : void
*/

static void dr_recoverScreenState(int status)
{
  if (currentmode.gcharaltered)
  { displ_setVDU5charsize(currentmode.gcharsizex,  currentmode.gcharsizey,
                          currentmode.gcharspacex, currentmode.gcharspacey);
    currentmode.gcharaltered = 0;
  }

/*  if (status & 0x20 == 0) bbc_vdu(4); */
}


#endif


/*
 Function    : dr_translateText
 Purpose     : apply font number translation to a diagram
 Parameters  : diagram structure
               translation table
 Returns     : void
*/


#ifdef BIGD

static void dr_translateText(Draw_diag diag, int * transTable)
{
  draw_objptr hdrptr;
  char *limit = diag.data + diag.length;

  for (hdrptr.bytep = diag.data+sizeof(draw_fileheader) ;
       hdrptr.bytep < limit ; hdrptr.bytep += hdrptr.objhdrp->size)
  {
    if (hdrptr.objhdrp->tag == draw_OBJTEXT)
    { /* Translate font number */
      hdrptr.textp->textstyle.fontref
        = transTable[hdrptr.textp->textstyle.fontref];
    }
  }
}


#endif

/*------------------------------ Interface functions ---------------------------*/

/*
 Function    : Draw_create_diag
 Purpose     : create an empty diagram
 Parameters  : pointer to diagram structure
               name of creating program
               bounding box for diagram
 Returns     : void
 Description : Create an empty diagram (ie just the file header), with the
               given bounding box, measured in Draw units. The memory for the
               diagram must have already been allocated. The length field in
               the diagram structure is set up. The first 12 characters of
               the creator name are also logged in the file header; the
               remaining fields are given standard values.
*/

#ifdef BIGD

void Draw_create_diag(Draw_diag *diag, char *creator, Draw_box bbox)
{
  draw_objptr hdrptr;

  /* Create file header */
  hdrptr.bytep = diag->data;
  strncpy(hdrptr.filehdrp->title, "Draw", 4);
  hdrptr.filehdrp->majorstamp = majorformatversionstamp;
  hdrptr.filehdrp->minorstamp = minorformatversionstamp;
  strncpy(hdrptr.filehdrp->progident, creator, 12);

  /* Copy box using a grotesque type fiddle */
  hdrptr.filehdrp->bbox = *((draw_bboxtyp*)&bbox);

  /* Record length field */
  diag->length = sizeof(draw_fileheader);
}


#endif

/*
 Function    : Draw_doObjects
 Purpose     : render a range of objects
 Parameters  : diagram structure
               handle for start object
               handle for end object
               pointer to redraw structure
               scale factor
               OUT: error block
 Returns     : TRUE if successful
 Description : Render the specified range of objects. The remaining parameters
               are used in the same way as Draw_render_diag() in level 0.
               As with Draw_render_diag the diagram should have been verified
               first.
*/

#ifdef BIGD

BOOL Draw_doObjects(Draw_diag * diag,Draw_object start,Draw_object end,
     Draw_redrawstr *r,transformstr * tr,processstr * pr,Draw_error *error)
{
  draw_objptr startObject, endObject;
  char       *limit=diag->data+diag->length;
  BOOL       err = FALSE;
  int        orgx, orgy;
  Draw_box   clipBox;
  int        vdu_status;
  coordstr   cscale;
  int        temp;


  drawtypemask=
   (!strncmp(((draw_fileheader*)diag->data)->progident,"DrawPlus",8) ||
    !strncmp(((draw_fileheader*)diag->data)->progident,"Vector",6))?0xFF:-1;


  /* Locate the objects */
  startObject.bytep = diag->data + dr_setStart(start);
  dr_setEnd(diag, end, &endObject);

  if (startObject.bytep < diag->data || startObject.bytep > limit)
  {
    DrawFile_Error(Draw_BadObjectHandle, start);
    return (FALSE);
  }

  if (endObject.bytep < diag->data || endObject.bytep > limit+1
      || endObject.bytep < startObject.bytep)
  {
    DrawFile_Error(Draw_BadObjectHandle, end);
    return (FALSE);
  }

  /* Find where to plot */
/*  dr_setBox(r, xscale, yscale, &orgx, &orgy, angle, &clipBox); */


  tr->e=orgx;
  tr->f=orgy;

  /* Draw the objects */
  vdu_status = dr_setScreenState();

  rotated=(tr_type(tr) & (TR_ROTATE | TR_SKEW));

  tr_getscale(tr,&cscale);
  temp=ttmul(cscale.x,cscale.y);
  if(!temp) temp=1;
  flatness=squareroot(scale(200*200*cvdu.deltax*cvdu.deltay,0x10000,temp),0);
  flatness/=2;

/*  if(pr->isprinter) flatness=0; */

/*  dprintf(2,"flatness=%d",flatness); */



  if(pr->isprinter) minwidth=0;
  else              minwidth=flatness;


  Draw_memoryError = FALSE; /* Only relevant for text areas */


  err=do_objects(startObject,endObject,tr,pr,
                          clipBox.x0,clipBox.y0,clipBox.x1,clipBox.y1,error);


  dr_recoverScreenState(vdu_status);

  if (Draw_memoryError)
  {
    DrawFile_Error(Draw_TextColMemory, 0)
    return (FALSE);
  }
  else
    return (err);
}


#endif



BOOL Draw_renderObjects(Draw_diag * diag,Draw_object start,Draw_object end,
     Draw_box * box,transformstr * tr,processstr * pr,Draw_error *error)
{
 draw_objptr startObject, endObject;
 char       *limit=diag->data+diag->length;
 BOOL       err = FALSE;
// int        vdu_status;
 coordstr   cscale;
 int        tempx;
 int        tempy;

/*
 {
  draw_fileheader * d=diag->data;
  dprintf(8,"x0=%x x1=%x y0=%d y1=%d",
             d->bbox.x0,d->bbox.x1,d->bbox.y0,d->bbox.y1);
 }
*/

/*  *((int*)diag->data)==0x43777244 */



 drawtypemask=
   (!strncmp(((draw_fileheader*)diag->data)->progident,"DrawPlus",8) ||
    !strncmp(((draw_fileheader*)diag->data)->progident,"Vector",6))?0xFF:-1;


  /* Locate the objects */
 startObject.bytep = diag->data + dr_setStart(start);
 dr_setEnd(diag, end, &endObject);

 if(startObject.bytep<diag->data || startObject.bytep>limit)
 {
  DrawFile_Error(Draw_BadObjectHandle,start);
  return(FALSE);
 }

 if(endObject.bytep<diag->data || endObject.bytep>limit+1
      || endObject.bytep<startObject.bytep)
 {
  DrawFile_Error(Draw_BadObjectHandle, end);
  return (FALSE);
 }


  /* Draw the objects */

// vdu_status=dr_setScreenState();

 rotated=(tr_type(tr) & (TR_ROTATE | TR_SKEW));

 tr_getscale(tr,&cscale);
 tempx=scale(180*200,0x10000,cscale.x*cvdu.xdpi);
 tempy=scale(180*200,0x10000,cscale.y*cvdu.ydpi);
 flatness=squareroot(tempx*tempy,0);


/*  dprintf(4,"flat=%d",flatness);  */
/* dprintf(5,"x0=%d x1=%d y0=%d y1=%d",box->x0,box->x1,box->y0,box->y1); */


 if(pr->isprinter) minwidth=0;
 else              minwidth=flatness;


 Draw_memoryError = FALSE; /* Only relevant for text areas */


 err=do_objects(startObject,endObject,tr,pr,
                          box->x0,box->y0,box->x1,box->y1,error);


// dr_recoverScreenState(vdu_status);

 if(Draw_memoryError)
 {
  DrawFile_Error(Draw_TextColMemory,0)
  return(FALSE);
 }
 else
  return (err);
}





/*
 Function    : Draw_setFontTable
 Purpose     : scan the diagram for a font table object
 Parameters  : diagram structure
 Returns     : void
 Description : Scan the diagram for a font table object and record it for a
               following call of Draw_doObjects. This is needed if
               Draw_doObjects is to work on a sequence of objects that
               includes text objects using anti-aliased fonts, but no font
               table object. The result of Draw_doObjects in this case is
               unpredictable if it is not called first (may cause an
               exception). The font table remains valid until either a
               different one is encounted during Draw_doObjects, or until
               Draw_render_diag is called, or until a different diagram is
               rendered.
*/



void Draw_setFontTable(Draw_diag * diag)
{
  draw_objptr hdrptr;

  hdrptr.bytep = 0; /* Dummy to reset fontcat */
  note_fontcat(hdrptr);

  if (dr_findFontTable(diag, &hdrptr)) note_fontcat(hdrptr);
}



/*
 Function    : Draw_verifyObject
 Purpose     : check the data for an existing object
 Parameters  : diagram structure
               object handle
               OUT: size of object
               OUT: error block
 Returns     : TRUE if successful
 Description : Check the data for an existing object. 'size', if non-NULL
               returns the amount of memory occupied by the object. Verifying
               an object also ensures that its bounding box is consistent
               with the data in it; if not, no error is reported; the box is
               made consistent instead. On an error, the location is relative
               to the start of the diagram.
*/



#ifdef BIGD

BOOL Draw_verifyObject(Draw_diag diag, Draw_object object, int *size,
                       Draw_error *error)
{
  int code, location;
  draw_objptr hdrptr;
  hdrptr.bytep = diag.data + object;
  *size = hdrptr.objhdrp->size;

  if (check_Draw_object(hdrptr, &code, &location))
    return (TRUE);
  else
  { DrawFile_Error(code, location - (int)diag.data);
    return (FALSE);
  }
}


#endif

/*
 Function    : Draw_createObject
 Purpose     : create an object
 Parameters  : pointer to diagram structure
               object data block
               handle of object to insert after
               flag: rebind diagram?
               OUT: error block
 Returns     : TRUE if successful
 Description : Create an object with the given data. The object is inserted
               in the diagram after the specified one, and the remaining data
               moved down.
               'after' may be Draw_FirstObject/Draw_LastObject for inserting at
               the start/end of the diagram. The diagram must be large enough
               for the new data; its length field is updated.
               The diagram bounding box (in the file header) is updated to the
               union of its existing value and that of the new object if the
               flag 'rebind' is TRUE.
               On an error, the location is not meaningful.
               The handle for the new object (or the merged font table) is
               returned in 'object'.
               If the routine is used to create a font table, 'after' is
               ignored, and the object merged with the existing one (if there
               is one) or inserted at the start of the diagram (if not). This
               can cause the font reference numbers to change; immediately
               following this routine by a call to Draw_translate text will
               apply the change (only needed if there are text objects in
               anti-aliased fonts.
*/


#ifdef BIGD

BOOL Draw_createObject(Draw_diag *diag, Draw_objectType newObject,
                       Draw_object after, BOOL rebind, Draw_object *object,
                       Draw_error *error)
{
  fontTableChanged = FALSE;

  /* If the object is a font table, merge it with the current one */
  if (newObject.object->tag == draw_OBJFONTLIST)
  { int i;
    draw_objptr fontTable1, fontTable2;

    for (i = 0 ; i < 256 ; i++) Draw_transTable[i] = 0;

    if (!dr_findFontTable(*diag, &fontTable1)) fontTable1.bytep = 0;
    fontTable2.bytep = newObject.bytep;
    if (dr_mergeFontTables(diag, fontTable1, *diag, fontTable2,
                           Draw_transTable, error))
    {
      *object = (fontTable1.bytep == 0) ? sizeof(draw_fileheader)
                                        : (int)(fontTable1.bytep - diag->data);
      fontTableChanged = TRUE;
      return (TRUE);
    }
    else
    {
      return (FALSE);
    }
  }
  else
  { char *to;

    /* Find where to insert object */
    if (after == Draw_LastObject)
      to = diag->data + diag->length;
    else
    {
      if (after == Draw_FirstObject)
        to = diag->data + sizeof(draw_fileheader);
      else
      { draw_objptr afterObject;
        afterObject.bytep = diag->data + after;
        to = afterObject.bytep + afterObject.objhdrp->size;
      }
    }

    /* Check the insert location is sensible */
    if (to < diag->data || to > diag->data + diag->length)
    {
      DrawFile_Error(Draw_BadObjectHandle, after);
      return (FALSE);
    }

    /* Move down rest of diagram, if necessary */
    if (to < diag->data + diag->length)
    {
      dr_spaceCopy(to+newObject.object->size, to, 
                   (int)(diag->data + diag->length - to));
    }

    /* Copy in the new data */
    memcpy(to, newObject.bytep, newObject.object->size);
    diag->length += newObject.object->size;
    *object = (int)(to - diag->data);

    /* Unify with diagram header box if asked */
    if (rebind)
    { draw_objptr fileHeader;

      fileHeader.bytep = diag->data;
      dr_unifyBoxes(&fileHeader.filehdrp->bbox, &newObject.object->bbox);
    }
  }

  return (TRUE);
}

#endif



/*
 Function    : Draw_deleteObjects
 Purpose     : delete a range of objects
 Parameters  : pointer to diagram structure
               handle of first object to delete
               handle of last object to delete
               flag: rebind diagram?
               OUT: error block
 Returns     : TRUE if successful
 Description : Delete the specified range of objects. If 'rebind' is TRUE, the
               diagram bounding box is set to the union of those of the
               remaining objects. The length field of the diagram is updated.
*/


#ifdef BIGD

BOOL Draw_deleteObjects(Draw_diag *diag, Draw_object start, Draw_object end,
                        BOOL rebind, Draw_error *error)
{
  draw_objptr startObject, endObject;
  char *limit = diag->data + diag->length;

  /* Locate first and last objects */
  startObject.bytep = diag->data + dr_setStart(start);
  dr_setEnd(*diag, end, &endObject);
  if (startObject.bytep < diag->data || startObject.bytep > limit)
  {
    DrawFile_Error(Draw_BadObjectHandle, start);
    return (FALSE);
  }

  if (endObject.bytep < diag->data || endObject.bytep > limit+1
      || endObject.bytep < startObject.bytep)
  {
    DrawFile_Error(Draw_BadObjectHandle, end);
    return (FALSE);
  }

  /* Close up memory */
  dr_spaceCopy(startObject.bytep, endObject.bytep,
               (int)(diag->data + diag->length - endObject.bytep));
  diag->length -= (int)(endObject.bytep - startObject.bytep);

  /* Rebind diagram if asked */
  if (rebind) Draw_rebind_diag(diag);

  return (TRUE);
}


#endif

/*
 Function    : Draw_extractObject
 Purpose     : extract the definition of an object
 Parameters  : diagram structure
               object handle
               object data block
               OUT: error block
 Returns     : TRUE if successful
 Description : Extract the definition of the given object into the buffer at
               'result', which must be large enough to contain it.
*/


#ifdef BIGD

BOOL Draw_extractObject(Draw_diag diag, Draw_object object,
                        Draw_objectType result, Draw_error *error)
{
  draw_objptr hdrptr;

  /* Find the object */
  hdrptr.bytep = diag.data + object;
  if (hdrptr.bytep < diag.data || hdrptr.bytep > diag.data + diag.length)
  {
    DrawFile_Error(Draw_BadObjectHandle, object)
    return (FALSE);
  }

  /* Copy out the data */
  dr_spaceCopy(result.bytep, hdrptr.bytep, hdrptr.objhdrp->size);
  return (TRUE);
}


#endif

/*
 Function    : Draw_translateText
 Purpose     : translate font references
 Parameters  : diagram structure
 Returns     : void
 Description : Updates all font reference numbers for text objects following
               creation of a font table. See Draw_createObject.
*/


#ifdef BIGD

void Draw_translateText(Draw_diag diag)
{
  if (fontTableChanged) dr_translateText(diag, Draw_transTable);
}

#endif

