/* sprite.h
 * Copyright (C) Acorn Computers Ltd., 1989
 * Copyright (C) Codemist Ltd., 1989
 */
#ifndef __sprite_h
#define __sprite_h

#ifndef __os_h
#include "os.h"
#endif

#ifndef __transform
#include "transform.h"
#endif

#ifndef __xprocess
#include "xprocess.h"
#endif

typedef enum
{
 sprite_nopalette = 0,
 sprite_haspalette = 1
}sprite_palflag;

typedef struct
{
 int xmag,ymag,xdiv,ydiv;
}sprite_factors;

typedef int sprite_pixtrans; 

typedef struct 
{
 int size;
 int number;
 int sproff;
 int freeoff;
}sprite_area;

typedef struct 
{
 int next; 
 char name[12]; 
 int width; 
 int height; 
 int lbit; 
 int rbit; 
 int image; 
 int mask; 
 int mode; 
 
 
}sprite_header;


#define SPT_CMYK 7



typedef struct /* Format of a sprite header */
{
  int  next;      /*  Offset to next sprite                */
  char name[12];  /*  Sprite name                          */
  int  width;     /*  Width in words-1      (0..639)       */
  int  height;    /*  Height in scanlines-1 (0..255/511)   */
  int  lbit;      /*  First bit used (left end of row)     */
  int  rbit;      /*  Last bit used (right end of row)     */
  int  image;     /*  Offset to sprite image               */
  int  mask;      /*  Offset to transparency mask          */

  union
  {
			struct
			{
    unsigned int mode:31;      /*  Mode sprite was defined in           */
		  unsigned int alpha:1;
			}	mode;

   struct
   {
    unsigned int newtype:1;   // can't use this on mode numbers i.e. doesn't help resolve new sprites, use type!=0
    unsigned int xdpi:13;
    unsigned int ydpi:13;
    unsigned int type:4;
		  unsigned int alpha:1;
   } type;

  } selector;



} newsprite_header;








#define sprite_mainarea ((sprite_area *) 0)

typedef void * sprite_ptr;

void sprite_area_initialise(sprite_area *, int size);


os_error * sprite_area_merge(sprite_area *, const char *filename);

os_error * sprite_area_merge_memory(sprite_area * area,char * merge);

newsprite_header * sprite_find(sprite_area * areap,char * name);


typedef enum
{
 sprite_id_name = 0,
 sprite_id_addr = 0x74527053 
}sprite_type;


typedef struct
{
 union
 {
 char * name; 
 sprite_ptr addr;
 } s;
 sprite_type tag; 
}sprite_id;



os_error * sprite_put_given(sprite_area *, sprite_id *, int gcol,
 int x, int y);


os_error * sprite_put_givenpx(sprite_area *area, sprite_id *spr, int gcol_action,
                            int x, int y);

os_error * sprite_put_scaled(sprite_area *, sprite_id *, int gcol,
 int x, int y,
 sprite_factors *factors,
 sprite_pixtrans * pixtrans);


os_error * sprite_put_scaledpx(sprite_area *area, sprite_id *spr,
                             int gcol_action,
                             int x, int y,
                             sprite_factors  *factors,
                             sprite_pixtrans *pixtrans);




typedef struct
{
 int width;
 int height;
 int mask;
 int mode;
}sprite_info;

os_error * sprite_readsize(sprite_area *, sprite_id *,
 sprite_info *resultinfo);





typedef struct 
{
 int a;
 int b;
 int c;
 int d;
 int e;
 int f;
} sprite_transform;


os_error * sprite_put_trans(sprite_area *area, sprite_id *spr,int gcol_action,
                         sprite_transform * trans,sprite_pixtrans *pixtrans);


os_error * sprite_put_transproc(sprite_area *area,sprite_id *spr,int gcol_action,
                            sprite_transform * trans,processstr * prc,sprite_pixtrans *pixtrans);


 
#endif
