/****************************************************************************
 * This source file was written by Acorn Computers Limited. It is part of   *
 * the RISCOS library for writing applications in C for RISC OS. It may be  *
 * used freely in the creation of programs for Archimedes. It should be     *
 * used with Acorn's C Compiler Release 3 or later.                         *
 *                                                                          *
 ***************************************************************************/

/* Title:   wimp.h
 * Purpose: C interface to RISC OS Wimp routines.
 *     
 */

# ifndef __wimp_h
# define __wimp_h


# ifndef __os_h
# include "os.h"
# endif

# ifndef __sprite_h
# include "sprite.h"
# endif


#ifndef __font_h
#include "font.h"
#endif


#ifndef BOOL
#define BOOL int
#define TRUE 1
#define FALSE 0
#endif


typedef enum
{

 wimp_NOCHANGE=  -3,
 wimp_TOP     =  -1,
 wimp_BOTTOM  =  -2,
 wimp_TOPMOST =  -4,
 wimp_NOTOPMOST= -5,
 wimp_HIDDEN  =  -6,
}	wimp_posn;


typedef enum {                     /* window flag set */

  wimp_WMOVEABLE = 0x00000002,     /* is moveable */
  wimp_REDRAW_OK = 0x00000010,     /* can be redrawn entirely by wimp
                                    * ie. no user graphics */
  wimp_WPANE     = 0x00000020,     /* window is stuck over tool window */
  wimp_WTRESPASS = 0x00000040,     /* window is allowed to go outside
                                    * main area */
  wimp_WSCROLL_R1= 0x00000100,     /* scroll request returned when
                                    * scroll button clicked- auto repeat*/
  wimp_SCROLL_R2 = 0x00000200,     /* as SCROLL_R1, debounced, no auto */
  wimp_REAL_COLOURS = 0x000000400, /* use real window colours. */
  wimp_BACK_WINDOW = 0x000000800,  /* this window is a background window. */
  wimp_HOT_KEYS = 0x000001000,     /* generate events for 'hot keys' */

  wimp_WOPEN  = 0x00010000,        /* window is open */
  wimp_WTOP   = 0x00020000,        /* window is on top (not covered) */
  wimp_WFULL  = 0x00040000,        /* window is full size */
  wimp_WCLICK_TOGGLE = 0x00080000, /* open_window_request was due to click
                                    * on toggle-size button */
  wimp_WFOCUS   = 0x00100000,      /* window has input focus */ 
  wimp_OPENING  = 0x00200000,      /* window is being opened */ 
  wimp_MINIMISED= 0x00400000,      /* window is minimised */ 

  wimp_WBACK  = 0x01000000,        /* window has back button */
  wimp_WQUIT  = 0x02000000,        /* has a quit button */
  wimp_WTITLE = 0x04000000,        /* has a title bar */
  wimp_WTOGGLE= 0x08000000,        /* has a toggle-size button */
  wimp_WVSCR  = 0x10000000,        /* has vertical scroll bar */
  wimp_WSIZE  = 0x20000000,        /* has size box */
  wimp_WHSCR  = 0x40000000,        /* has horizontal scroll bar */
  wimp_EXISTS = 0x80000000,        /* has been completely created */
  wimp_WNEW   = ~0x7fffffff        /* use these new flags */

                                   /* NB! always set the WNEW flag */
} wimp_wflags;

typedef enum {
  wimp_WCTITLEFORE,
  wimp_WCTITLEBACK,
  wimp_WCWKAREAFORE,
  wimp_WCWKAREABACK,
  wimp_WCSCROLLOUTER,
  wimp_WCSCROLLINNER,
  wimp_WCTITLEHI,
  wimp_WCRESERVED
} wimp_wcolours;
/* If work area background is 255 then it isn't painted. */
/* If title foreground is 255 then you get no borders, title etc. at all */

typedef enum {                      /* icon flag set */
  wimp_ITEXT      = 0x00000001,     /* icon contains text */
  wimp_ISPRITE    = 0x00000002,     /* icon is a sprite */
  wimp_IBORDER    = 0x00000004,     /* icon has a border */
  wimp_IHCENTRE   = 0x00000008,     /* text is horizontally centred */
  wimp_IVCENTRE   = 0x00000010,     /* text is vertically centred */
  wimp_IFILLED    = 0x00000020,     /* icon has a filled background */
  wimp_IFONT      = 0x00000040,     /* text is an anti-aliased font */
  wimp_IREDRAW    = 0x00000080,     /* redraw needs application's help */
  wimp_INDIRECT   = 0x00000100,     /* icon data is 'indirected' */
  wimp_IRJUST     = 0x00000200,     /* text right justified in box */
  wimp_IESG_NOC   = 0x00000400,     /* if selected by right button, don't
                                     * cancel other icons in same ESG */
  wimp_IHALVESPRITE=0x00000800,     /* plot sprites half-size */
  wimp_IBTYPE     = 0x00001000,     /* 4-bit field: button type */
  wimp_ISELECTED  = 0x00200000,     /* icon selected by user (inverted) */
  wimp_INOSELECT  = 0x00400000,     /* icon cannot be selected (shaded) */
  wimp_IDELETED   = 0x00800000,     /* icon has been deleted */
  wimp_IFORECOL   = 0x01000000,     /* 4-bit field: foreground colour */
  wimp_IBACKCOL   = 0x10000000      /* 4-bit field: background colour */
} wimp_iconflags;

/* If the icon contains anti-aliased text then the colour fields
give the font handle: */
#define wimp_IFONTH 0x01000000

typedef enum {                /* button types */
  wimp_BIGNORE,               /* ignore all mouse ops */
  wimp_BNOTIFY,
  wimp_BCLICKAUTO,
  wimp_BCLICKDEBOUNCE,
  wimp_BSELREL,
  wimp_BSELDOUBLE,
  wimp_BDEBOUNCEDRAG,
  wimp_BRELEASEDRAG,
  wimp_BDOUBLEDRAG,
  wimp_BSELNOTIFY,
  wimp_BCLICKDRAGDOUBLE,
  wimp_BCLICKSEL,              /* useful for on/off and radio buttons */
  wimp_BWRITECLICKDRAG=14,
  wimp_BWRITABLE = 15
} wimp_ibtype;

typedef enum {                 /* button state bits */
  wimp_BRIGHT       = 0x001,
  wimp_BMID         = 0x002,
  wimp_BLEFT        = 0x004,
  wimp_BDRAGRIGHT   = 0x010,
  wimp_BDRAGLEFT    = 0x040,
  wimp_BCLICKRIGHT  = 0x001,
  wimp_BCLICKLEFT   = 0x004,
  wimp_DOUBLERIGHT  = 0x100,
  wimp_DOUBLELEFT   = 0x400,
  wimp_RELEASERIGHT = 0x1000,
  wimp_RELEASELEFT  = 0x4000,

} wimp_bbits;

typedef enum {
  wimp_MOVE_WIND = 1,                 /* change position of window */
  wimp_SIZE_WIND = 2,                 /* change size of window */
  wimp_DRAG_HBAR = 3,                 /* drag horizontal scroll bar */
  wimp_DRAG_VBAR = 4,                 /* drag vertical scroll bar */
  wimp_USER_FIXED = 5,                /* user drag box - fixed size */
  wimp_USER_RUBBER = 6,               /* user drag box - rubber box */
  wimp_USER_HIDDEN = 7,               /* user drag box - invisible box */
  wimp_USER_SPLITTER=8,               /* splitter bar                  */

} wimp_dragtype;

/*****************************************************************************/

#ifdef _WIN64

typedef HWND wimp_w; /* abstract window handle */


#else

typedef int wimp_w; /* abstract window handle */



#endif

typedef int wimp_i; /* abstract icon handle */

typedef int wimp_t; /* abstract task handle */




typedef union {           /* the data field in an icon */
  char text[12];          /* up to 12 bytes of text */
  char sprite_name[12];   /* up to 12 bytes of sprite name */
  struct {
    char *name;
    void *spritearea;     /* 0->use the common sprite area */
                          /* 1->use the wimp sprite area */
    BOOL nameisname;      /* if FALSE, name is in fact a sprite pointer. */
  } indirectsprite;
  struct {                /* if indirect */
    char *buffer;         /* pointer to text buffer */
    char *validstring;    /* pointer to validation string */
    int bufflen;          /* length of text buffer */
  } indirecttext;
} wimp_icondata;

typedef struct {
  int x0, y0, x1, y1;
} wimp_box;




#ifdef NEVER

typedef struct {
  wimp_box box;           /* screen coords of work area */
  int scx, scy;           /* scroll bar positions */
  wimp_w behind;          /* handle to open window behind, or -1 if top */
  wimp_wflags flags;      /* word of flag bits defined above */
  char colours[8];        /* colours: index using wimp_wcolours. */
  wimp_box ex;            /* maximum extent of work area */
  wimp_iconflags titleflags;     /* icon flags for title bar */
  wimp_iconflags workflags;      /* just button type relevant */
  void *spritearea;       /* 0->use the common sprite area */
                          /* 1->use the wimp sprite area */
  int minsize;            /* two 16-bit OS-unit fields, (width/height)
                             giving min size of window. 0->use title. */
  wimp_icondata title;    /* title icon data */
  int nicons;             /* no. of icons in window */
} wimp_wind;
/* If there are any icon definitions, they should follow this structure
immediately in memory. */

typedef struct {          /* result of get_info call. */
  wimp_w w;
  wimp_wind info;
} wimp_winfo;
/* Space for icons must follow. */

#endif


typedef struct {

} wimp_wind;



typedef struct {          /* icon description structure */
  wimp_box box;           /* bounding box - relative to
                           * window origin (work area top left) */
  wimp_iconflags flags;   /* word of flag bits defined above */
  wimp_icondata data;     /* union of bits & bobs as above */
} wimp_icon;

typedef struct {          /* structure for creating icons. */
  wimp_w w;
  wimp_icon i;
} wimp_icreate;

typedef struct {
  wimp_w w;               /* window handle */
  wimp_box box;           /* position on screen of visible work area */
  int x, y;               /* 'real' coordinates of visible work area */
  wimp_w behind;          /* handle of window to go behind (-1 = top,
                           * -2 = bottom) */
} wimp_openstr;


typedef struct {

 wimp_openstr o;
 int          reason;
 LPARAM       lparam;
 WPARAM       wparam;

} wimp_openstr2;




typedef struct { /* result for window state enquiry */
  wimp_openstr o;
  wimp_wflags flags;
} wimp_wstate;

typedef enum {               /* event types */
  wimp_ENULL,                /* null event */
  wimp_EREDRAW,              /* redraw event */
  wimp_EOPEN,
  wimp_ECLOSE,
  wimp_EPTRLEAVE,
  wimp_EPTRENTER,
  wimp_EBUT,                 /* mouse button change */
  wimp_EUSERDRAG,
  wimp_EKEY,
  wimp_EMENU,
  wimp_ESCROLL,
  wimp_ELOSECARET,
  wimp_EGAINCARET,
  wimp_ESEND = 17,        /* send message, don't worry if it doesn't arrive */
  wimp_ESENDWANTACK = 18, /* send message, return ack if not acknowledged */
  wimp_EACK = 19          /* acknowledge receipt of message. */
} wimp_etype;

typedef enum {               /* event type masks */
  wimp_EMNULL     = 1 << wimp_ENULL,
  wimp_EMREDRAW   = 1 << wimp_EREDRAW,
  wimp_EMOPEN     = 1 << wimp_EOPEN,
  wimp_EMCLOSE    = 1 << wimp_ECLOSE,
  wimp_EMPTRLEAVE = 1 << wimp_EPTRLEAVE,
  wimp_EMPTRENTER = 1 << wimp_EPTRENTER,
  wimp_EMBUT      = 1 << wimp_EBUT,
  wimp_EMUSERDRAG = 1 << wimp_EUSERDRAG,
  wimp_EMKEY      = 1 << wimp_EKEY,
  wimp_EMMENU     = 1 << wimp_EMENU,
  wimp_EMSCROLL   = 1 << wimp_ESCROLL,
  wimp_EMLOSECARET= 1 << wimp_ELOSECARET,
  wimp_EMGAINCARET= 1 << wimp_EGAINCARET,
  wimp_EMSEND     = 1 << wimp_ESEND,
  wimp_EMSENDWANTACK
                  = 1 << wimp_ESENDWANTACK,
  wimp_EMACK      = 1 << wimp_EACK
} wimp_emask;

typedef struct {
  wimp_w w;
  wimp_box box;           /* work area coordinates */
  int scx, scy;           /* scroll bar positions */
  wimp_box g;             /* current graphics window */
  oshdc hdcptr;        /* handle to our graphics device context */
} wimp_redrawstr;

typedef struct {
  int x, y;         /* mouse x and y */
  wimp_bbits bbits; /* button state */
  wimp_w w;         /* window handle, or -1 if none */
  wimp_i i;         /* icon handle, or -1 if none */
} wimp_mousestr;

typedef struct {
  wimp_w w;
  wimp_i i;
  int x, y;                /* offset relative to window origin */
  int height;              /* -1 if calc within icon
                            * bit 24 -> VDU-5 type caret
                            * bit 25 -> caret invisible
                            * bit 26 -> bits 16..23 contain colour
                            * bit 27 -> colour is "real" colour */
  int startindex;          /* position within icon */
  int endindex;

} wimp_caretstr;

/* Message action codes are allocated just like SWI codes. */
typedef enum {
  wimp_MCLOSEDOWN    = 0, /* Reply if any dialogue with the user is required,
                             and the closedown sequence will be aborted. */
  wimp_MDATASAVE     = 1, /* request to identify directory */
  wimp_MDATASAVEOK   = 2, /* reply to message type 1 */
  wimp_MDATALOAD     = 3, /* request to load/insert dragged icon */
  wimp_MDATALOADOK   = 4, /* reply that file has been loaded */
  wimp_MDATAOPEN     = 5, /* warning that an object is to be opened */
  wimp_MRAMFETCH     = 6, /* transfer data to buffer in my workspace */
  wimp_MRAMTRANSMIT  = 7, /* I have transferred some data to a buffer in your
                             workspace */
  wimp_MPREQUIT      = 8,
  wimp_PALETTECHANGE = 9,
  wimp_SAVEDESK      = 10, /* Wimp 2.43 onwards - desktop closedown message */
  wimp_MDEVICECLAIM  = 11, /* Broadcast before an application can claim parallel port, RS232 port etc. */
  wimp_MDEVICEINUSE  = 12, /* Reply if another application is already using the device */
  wimp_MDATASAVED    = 13, /* A file previously saved has become 'safe' */

  wimp_FilerOpenDir  = 0x0400,
  wimp_FilerCloseDir = 0x0401,

  wimp_Notify        = 0x40040,  /* net filer notify broadcast */

  wimp_MMENUWARN     = 0x400c0,
  /* menu warning. Sent if wimp_MSUBLINKMSG set. Data sent is:
         submenu field of relevant wimp_menuitem.
         screen x-coord
         screen y-coord
         list of menu selection indices (0..n-1 for each menu)
         terminating -1 word.
     Typical response is to call wimp_create_submenu.
  */
  wimp_MMODECHANGE   = 0x400c1,
  wimp_MINITTASK     = 0x400c2,
  wimp_MCLOSETASK    = 0x400c3,
  wimp_MSLOTCHANGE   = 0x400c4,       /* Slot size has altered */
  wimp_MSETSLOT      = 0x400c5,       /* Task manager requests application
                                         to change its slot size */
  wimp_MTASKNAMERQ   = 0x400c6,       /* Request task name */
  wimp_MTASKNAMEIS   = 0x400c7,       /* Reply to task name request */
  wimp_MTASKSTARTED  = 0x400c8,       /* Broadcast by task to indicate that it has now fully started */

  wimp_MMENUDELETED  = 0x400C9,

  wimp_MHELPREQUEST  = 0x502,         /* interactive help request */
  wimp_MHELPREPLY    = 0x503,         /* interactive help message */

  /* Messages for dialogue with printer applications */

  wimp_MPrintFile       = 0x80140,    /* Printer app's first response to */
                                      /* a DATASAVE */
  wimp_MWillPrint       = 0x80141,    /* Acknowledgement of PrintFile */
  wimp_MPrintSave       = 0x80142,
  wimp_MPrintInit       = 0x80143,    /* Print manager starts up      */
  wimp_MPrintTypeOdd    = 0x80145,    /* Broadcast when strange files */
                                      /* dropped on the printer */
  wimp_MPrintTypeKnown  = 0x80146,    /* Ack to above */
  wimp_MPrinterChange   = 0x80147,    /* New printer application installed */

  wimp_MFONTCHANGE      = 0x88000,    /* Font changed, made up */

} wimp_msgaction;

typedef struct {          /* message block header. */
  int size;               /* 20<=size<=256, multiple of 4 */
  wimp_t task;            /* task handle of sender (filled in by wimp) */
  int my_ref;             /* unique ref number (filled in by wimp) */
  int your_ref;           /* (0==>none) if non-zero, acknowledge */
  wimp_msgaction action;  /* message action code */
} wimp_msghdr;
/* size is the size of the whole msgstr, see below. */




typedef struct {         /* structure used in all print messages */
  int filler[5] ;
  int type ;             /* filetype */
  char * name;           /* filename */
  char * printer;        // printer name or null
} wimp_msgprint ;

typedef struct {          /* message block */
  wimp_msghdr hdr;
  union {
    char chars[236];
    int words[59];             /* max data size. */
    wimp_msgprint       print;
  } data;
} wimp_msgstr;


typedef union {
    wimp_openstr o;         /* for redraw, close, enter, leave events */
    struct {
      wimp_mousestr m;
      wimp_bbits b;} but;   /* for button change event */
    wimp_box dragbox;       /* for user drag box event */
    struct {wimp_caretstr c; int chcode;} key;  /* for key events */
    int menu[10];           /* for menu event: terminated by -1 */
    struct {wimp_openstr o; int x, y;} scroll;  /* for scroll request */
                                       /* x=-1 for left, +1 for right */
                                       /* y=-1 for down, +1 for up    */
                             /* scroll by +/-2 -> page scroll request */
    wimp_caretstr c;        /* for caret gain/lose. */
    wimp_msgstr msg;        /* for messages. */
} wimp_eventdata;

typedef struct {            /* wimp event description */
  wimp_etype e;             /* event type */
  wimp_eventdata data;
} wimp_eventstr;

typedef struct {
  char title[12];         /* menu title (optional) */
  char tit_fcol, tit_bcol, work_fcol, work_bcol; /* colours */
  int width, height;      /* size of following menu items */
  int gap;                /* vertical gap between items */
} wimp_menuhdr;

typedef enum {           /* menu item flag set */
  wimp_MTICK       = 1,
  wimp_MSEPARATE   = 2,
  wimp_MWRITABLE   = 4,
  wimp_MSUBLINKMSG = 8, /* show a => flag, and inform program when it
                           is activated. */
  wimp_MLAST = 0x80     /* signal last item in the menu */
} wimp_menuflags;
/* use wimp_INOSELECT to shade the item as unselectable,
and the button type to mark it as writable. */

typedef struct wimp_menustr *wimp_menuptr;
/* Only for the circular reference in menuitem/str. */

typedef struct {
  wimp_menuflags flags;         /* menu entry flags */
  wimp_menuptr submenu;         /* wimp_menustr* pointer to sub menu,
                                   or wimp_w dialogue box,
                                   or -1 if no submenu */
  wimp_iconflags iconflags;     /* icon flags for the entry */
  wimp_icondata data;           /* icon data for the entry */
} wimp_menuitem;
/* submenu can also be a wimp_w, in which case the window is opened as a
dialogue box within the menu tree. */

typedef struct wimp_menustr {
  wimp_menuhdr hdr;
  /* wimp_menuitem item[];*/ /* Zero or more menu items follow in memory */
} wimp_menustr;

typedef struct {
  wimp_w window;
  wimp_dragtype type;
  wimp_box box;           /* initial position for drag box */
  wimp_box parent;        /* parent box for drag box */
} wimp_dragstr;




typedef union {
  struct {char gcol; char red; char green; char blue;} bytes;
  int word;
} wimp_paletteword;
/* The gcol char (least significant) is a gcol colour except in 8-bpp
modes, when bits 0..2 are the tint and bits 3..7 are the gcol colour. */

typedef struct {
  wimp_paletteword c[16];              /* wimp colours 0..15 */
  wimp_paletteword screenborder, mouse1, mouse2, mouse3;
} wimp_palettestr;

/*****************************************************************************/


os_error *wimp_create_wind(wimp_wind *, wimp_w *);
/* define (but not display) window, return window handle */

os_error *wimp_create_icon(wimp_icreate *, wimp_i *result);
/* add icon definition to that of window, return icon handle */

os_error *wimp_delete_wind(wimp_w);

os_error *wimp_delete_icon(wimp_w, wimp_i);

os_error *wimp_open_wind(wimp_openstr *);
/* make window appear on screen */

os_error * wimp_open_wind_ex(wimp_openstr * o);
/* as above but o contains external dimensions of window */

os_error *wimp_close_wind(wimp_w);
/* Remove from active list the window with handle in integer argument. */

os_error *wimp_redraw_wind(wimp_redrawstr*, BOOL* /*out*/);
/* Draw window outline and icons. Return FALSE if there's nothing to draw. */

os_error *wimp_update_wind(wimp_redrawstr*, BOOL* /*out*/);
/* Return visible portion of window. Return FALSE if nothing to redraw. */

os_error *wimp_get_rectangle(wimp_redrawstr*, BOOL*);
/* return next rectangle in list, or FALSE if done. */

os_error *wimp_get_wind_state(wimp_w, wimp_wstate * result);
/* read current window state */

//os_error * wimp_get_os_wind_state(wimp_w w, wimp_wstate * result);
///* read current window state - but not one of our windows */


os_error *wimp_set_icon_state(wimp_w, wimp_i,
  wimp_iconflags value, wimp_iconflags mask);
/* Set icon's flags as (old_state & ~mask) ^ value */

os_error *wimp_get_icon_info(wimp_w, wimp_i, wimp_icon * result);
/* Get current state of icon */

os_error *wimp_get_point_info(wimp_mousestr * result);
/* Give info regarding the state of the mouse */

os_error *wimp_drag_box(wimp_dragstr *);
/* start the wimp dragging a box */

os_error *wimp_force_redraw(wimp_redrawstr * r);
/* Mark an area of the screen as invalid.
If r->wimp_w == -1 then use screen coordinates. Only the first
five fields of r are valid. */

os_error *wimp_set_caret_pos(wimp_caretstr *);
/* set pos./size of text caret */

os_error *wimp_get_caret_pos(wimp_caretstr *);
/* get pos./size of text caret */


os_error *wimp_set_extent(wimp_redrawstr *);
/* Alter extent of a window's work area - only handle and 1st set of
4 coordinates are looked at. */

os_error * wimp_get_extent(wimp_redrawstr * wr);


os_error *wimp_getwindowoutline(wimp_redrawstr *r);
/* set r->w on entry. On exit, r->box will be the screen coordinates
of the window, including border, title, scroll bars. */

os_error *wimp_readpalette(wimp_palettestr*);


os_error *wimp_setcolour(int colour);
/* bits 0..3 = wimp colour (translate for current mode)
        4..6 = gcol action
        7    = foreground/background.
*/





os_error * wimp_create_windparent(wimp_wind * w, wimp_w * result,wimp_w parent);


os_error * wimp_setwindowbackground(wimp_w window,int colour);


#define WINDOWPANE          0x8
#define WINDOWFIXED         0x10
#define WINDOWNOTDIALOGUE   0x20
#define WINDOWNOSCROLLCHILD 0x40
#define WINDOWBEINGOPENED   0x80
#define WINDOWMINMAX        0x100
#define WINDOWAUTOCENTRE    0x200
#define WINDOWDDE           0x400
#define WINDOWUNICODE       0x800



os_error * wimp_setwindowflags(wimp_w window,int orr,int bic);
os_error * wimp_getwindowflags(wimp_w window,int * flags);


os_error * wimp_getfont(wimp_w hwnd,font * fh);
os_error * wimp_setfont(wimp_w hwnd,font fh);


os_error * wimp_copy_icon_info(wimp_icon ** dest,wimp_icon * src);
os_error * wimp_lose_icon_info(wimp_icon ** icon);

os_error * wimp_ploticon(wimp_w hwnd,wimp_icon * icon,sprite_area * spa);

os_error * wimp_set_icon_colour(wimp_w w,wimp_i i,int fcol,int bcol);

#define WMOVEICONSIZE 0x1

os_error * wimp_move_icon(wimp_w handle,int icon,wimp_icon * i,int flags);

os_error * wimp_slice_icon(wimp_w w,wimp_i i,int ctype,char * newparam);

os_error * wimp_read_slice(wimp_w w,wimp_i i,int itype,int  * present,char * buffer,int * buffsize);

extern void wimp_check(char * string);

int wimp_wheelgetlines(wimp_w handle,int delta,int horizontal);


# endif

/* end wimp.h */
